/**
    (C) Copyright 2012 Dominik Wagenfuehr

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef STRATEGYFACTORY_HH
#define STRATEGYFACTORY_HH

#include <string>
#include <map>

// Vorwaertsdeklarationen.
class IStrategy;

/// Erstellt eine bestimmte Strategie.
/**
 * Dies ist eine sehr simple Factory, welche eine bestimmte
 * Strategie erstellt, je nachdem welcher String als Identifier
 * uebergeben wird.
 */
class StrategyFactory
{
  public:
    /// Gibt Instanz der Klasse zurueck.
    static StrategyFactory& getInstance();

    /// Registriert eine Strategie.
    /**
     * @param id Strategie-Identifikator
     * @param baseP Vorlage-Objekt für die Strategie
     * @return true, wenn Strategie registriert wurde,
     * oder false bei Fehler
     */
    bool registerStrategy( const std::string& id, IStrategy* baseP );

    // Hinweis: Die Methode darf nicht "register" genannt werden,
    // da dies ein vorbelegtes Wort in C++ ist.

    /// Erstellt eine bestimmte Strategie.
    /**
     * @param id Strategie-Identifikator
     * @return konkrete Strategie als Basis-Pointer oder 0 bei
     * einem Fehler
     */
    IStrategy* create( const std::string& id );

  private:
    /// Konstruktor.
    /**
     * Der Konstruktor ist privat, damit niemand sonst eine Instanz
     * der Klasse anlegen kann, ausser die Klasse selbst.
     */
    StrategyFactory() { };

    /// Copy-Konstruktor.
    /**
     * Der Konstruktor wird nicht implementiert, da niemand eine
     * Kopie anlegen soll.
     */
    StrategyFactory( const StrategyFactory& );

  private:
    /// Assoziatives Array mit allen moeglichen Strategien.
    /**
     * Das Array wird in der Implementierung befuellt und muss
     * bei neuen Strategien einfach nur erweitert werden.
     */
    std::map<std::string,IStrategy*> mStrategies;
};

#endif // STRATEGYFACTORY_HH
